<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Http\Requests\Auth\ConfirmVerificationCodeRequest;
use App\Http\Requests\Auth\LoginRequest;
use App\Http\Requests\Auth\RegisterRequest;
use App\Http\Requests\Auth\ResendVerificationCodeRequest;
use App\Services\Auth\AuthService;
use App\Services\User\UserService;
use App\Services\Verification\VerificationService;
use Exception;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Contracts\View\View;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;


class AuthController extends Controller
{
    protected AuthService $authService;
    protected VerificationService $verificationService;
    protected UserService $userService;

    /**
     * @param AuthService $authService
     * @param VerificationService $verificationService
     * @param UserService $userService
     */
    public function __construct(AuthService $authService, VerificationService $verificationService, UserService $userService)
    {
        $this->authService = $authService;
        $this->verificationService = $verificationService;
        $this->userService = $userService;
//        $this->middleware('auth:sanctum')->only('logout');
    }

    /**
     * register user function
     *
     * @param RegisterRequest $request
     * @return JsonResponse
     * @throws Exception
     */
    public function register(RegisterRequest $request): JsonResponse
    {
        $this->authService->register($request->validated());
        return success(msg: 'user registered , check your email for verification code  ');
    }


    public function login(LoginRequest $request)
    {
        $token =$this->authService->login($request->validated());
        $expiration = config('sanctum.custom-expiration');

        return loggedInSuccessfully(
            token: $token,
            expiresIn: "$expiration[0] $expiration[1]",
        );
    }

    /**
     * @throws Exception
     */
    public function resendVerificationCode(ResendVerificationCodeRequest $request): JsonResponse
    {
        $user = $this->userService->getUserByEmail($request->validated()['email']);
        $this->userService->throwIfUserVerified($user);
        $this->verificationService->sendVerificationCodeViaEmail($user);
        return success(msg: 'verification code sent , check your email for verification code ');
    }

    /**
     * confirm the verification code for the user and give it token
     *
     * @param ConfirmVerificationCodeRequest $request
     * @return JsonResponse
     */
    public function confirmVerificationCode(ConfirmVerificationCodeRequest $request): JsonResponse
    {
        $token = $this->verificationService->confirmVerificationCode(
            $request->validated(),
        );
        return loggedInSuccessfully($token, 'user logged in successfully');
    }


    public function confirmVerificationLink(Request $request)
    {

        $email = decrypt($request->email);
        $token = $this->verificationService->confirmVerificationLink($email, $email);
        return ' here i need to return link form the front end and the link must contain token ' . $token;
    }

    public function logout(): JsonResponse
    {
        auth()->user()->currentAccessToken()->delete();

        return success(
            msg: 'Logged out successfully'
        );
    }

}
