<?php

namespace App\Services\Verification;

use App\Models\User;
use App\Services\Auth\AuthService;
use App\Services\Mail\MailService;
use App\Services\Else\RandomStringGeneratorService;
use App\Services\User\UserService;
use Exception;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\URL;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;


class VerificationService
{

    /**
     * @var MailService
     */
    protected MailService $mailService;
    /**
     * @var RandomStringGeneratorService
     */
    protected RandomStringGeneratorService $randomStringGeneratorService;
    /**
     * @var UserService
     */
    protected UserService $userService;
    /**
     * @var AuthService
     */
    protected AuthService $authService;

    /**
     * @param MailService $mailService
     * @param RandomStringGeneratorService $randomStringGeneratorService
     * @param UserService $userService
     * @param AuthService $authService
     */
    public function __construct(
        MailService                  $mailService,
        RandomStringGeneratorService $randomStringGeneratorService,
        UserService                  $userService,
        AuthService                  $authService
    )
    {
        $this->mailService = $mailService;
        $this->randomStringGeneratorService = $randomStringGeneratorService;
        $this->userService = $userService;
        $this->authService = $authService;
    }

    /**
     * @throws Exception
     */
    public function sendVerificationCodeViaEmail(User $user): void
    {
        $verificationCode = $this->generateVerificationCode();
        $this->storeVerificationCodeInCache($verificationCode, $user->email);
        $verificationUrl = $this->generateVerificationUrl($user->email, 'verification.verify');
        $this->mailService->sendVerificationMail("verification email ",$verificationUrl,  $user, 'verify email ', $verificationCode);
    }

    /**
     * @param string $verificationCode
     * @param $userEmail
     * @return void
     */
    public function storeVerificationCodeInCache(string $verificationCode, $userEmail): void
    {
        Cache::put(
            $userEmail,
            encrypt($verificationCode),
            now()->addMinutes(10)
        );
    }

    /**
     * @param string $userEmail
     * @param string $redirectToRouteName
     * @return string
     */
    public function generateVerificationUrl(string $userEmail, string $redirectToRouteName): string
    {
        return URL::temporarySignedRoute(
            $redirectToRouteName,
            now()->addMinutes(30),
            ['email' => encrypt($userEmail)],
        );
    }

    /**
     * @return string
     * @throws Exception
     */
    public function generateVerificationCode(): string
    {
        return $this->randomStringGeneratorService->generate(6);
    }

    /**
     * @param $data
     * @param $expireTime
     * @return string
     */
    public function confirmVerificationCode($data): string
    {
        $user = $this->userService->getUserByEmail($data["email"]);
        $this->userService->throwIfUserVerified($user);
        $this->validateVerificationCode($data['email'], $data["code"]);
        Cache::forget($data["email"]);
        $user->email_verified_at = now();
        $user->save();
        return $this->authService->createToken($user);
    }

    /**
     * @param $email
     * @param null $forgetCacheData
     * @return string
     */
    public function confirmVerificationLink($email, $forgetCacheData = null): string
    {
        $user = $this->userService->getUserByEmail($email);
        if ($forgetCacheData !== null) {
            $this->userService->throwIfUserVerified($user);
            Cache::forget($forgetCacheData);
            $user->email_verified_at = now();
            $user->save();
        }
        return $this->authService->createToken($user);
    }


    /**
     * @param $userEmail
     * @param $verificationCode
     * @return void
     */
    private function validateVerificationCode($userEmail, $verificationCode): void
    {
        $cachedHashedCode = Cache::get($userEmail);
        if (!$cachedHashedCode)
            throw new BadRequestHttpException('Verification code has expired .');
        $cachedHashedCode = decrypt($cachedHashedCode);
        if ($verificationCode !== $cachedHashedCode)
            throw new BadRequestHttpException('Invalid verification code.');
    }
}
